"""
Type annotations for botocore.retries.adaptive module.

Copyright 2025 Vlad Emelianov
"""

from logging import Logger
from typing import Any

from botocore.client import BaseClient
from botocore.retries import bucket as bucket
from botocore.retries import standard as standard
from botocore.retries import throttling as throttling

logger: Logger = ...

def register_retry_handler(client: BaseClient) -> ClientRateLimiter: ...

class ClientRateLimiter:
    def __init__(
        self,
        rate_adjustor: Any,
        rate_clocker: Any,
        token_bucket: Any,
        throttling_detector: Any,
        clock: Any,
    ) -> None: ...
    def on_sending_request(self, request: Any, **kwargs: Any) -> None: ...
    def on_receiving_response(self, **kwargs: Any) -> None: ...

class RateClocker:
    def __init__(self, clock: Any, smoothing: Any = ..., time_bucket_range: Any = ...) -> None: ...
    def record(self, amount: int = ...) -> Any: ...
    @property
    def measured_rate(self) -> Any: ...



"""
Type annotations for botocore.config module.

Copyright 2025 Vlad Emelianov
"""

from typing import Any, Literal, Mapping, TypedDict, TypeVar

from botocore.compat import OrderedDict as OrderedDict
from botocore.endpoint import DEFAULT_TIMEOUT as DEFAULT_TIMEOUT
from botocore.endpoint import MAX_POOL_CONNECTIONS as MAX_POOL_CONNECTIONS
from botocore.exceptions import InvalidMaxRetryAttemptsError as InvalidMaxRetryAttemptsError
from botocore.exceptions import InvalidRetryConfigurationError as InvalidRetryConfigurationError
from botocore.exceptions import InvalidRetryModeError as InvalidRetryModeError
from botocore.exceptions import InvalidS3AddressingStyleError as InvalidS3AddressingStyleError

class _RetryDict(TypedDict, total=False):
    total_max_attempts: int
    max_attempts: int
    mode: Literal["legacy", "standard", "adaptive"]

class _S3Dict(TypedDict, total=False):
    use_accelerate_endpoint: bool
    payload_signing_enabled: bool
    addressing_style: Literal["auto", "virtual", "path"]
    us_east_1_regional_endpoint: Literal["regional", "legacy"]

class _ProxiesConfigDict(TypedDict, total=False):
    proxy_ca_bundle: str
    proxy_client_cert: str | tuple[str, str]
    proxy_use_forwarding_for_https: bool

_Config = TypeVar("_Config", bound=Config)

class Config:
    OPTION_DEFAULTS: OrderedDict[str, None]
    NON_LEGACY_OPTION_DEFAULTS: dict[str, None]
    def __init__(
        self,
        region_name: str | None = None,
        signature_version: str | None = None,
        user_agent: str | None = None,
        user_agent_extra: str | None = None,
        connect_timeout: float | None = 60,
        read_timeout: float | None = 60,
        parameter_validation: bool | None = True,
        max_pool_connections: int | None = 10,
        proxies: Mapping[str, str] | None = None,
        proxies_config: _ProxiesConfigDict | None = None,
        s3: _S3Dict | None = None,
        retries: _RetryDict | None = None,
        client_cert: str | tuple[str, str] | None = None,
        inject_host_prefix: bool | None = True,
        endpoint_discovery_enabled: bool | None = None,
        use_dualstack_endpoint: bool | None = None,
        use_fips_endpoint: bool | None = None,
        defaults_mode: bool | None = None,
        tcp_keepalive: bool | None = False,
        request_min_compression_size_bytes: int | None = None,
        disable_request_compression: bool | None = None,
        sigv4a_signing_region_set: str | None = None,
        client_context_params: Mapping[str, Any] | None = None,
        request_checksum_calculation: Literal["when_supported", "when_required"] | None = None,
        response_checksum_validation: Literal["when_supported", "when_required"] | None = None,
        account_id_endpoint_mode: Literal["preferred", "disabled", "required"] | None = None,
    ) -> None: ...
    def merge(self: _Config, other_config: _Config) -> _Config: ...
    @property
    def inject_host_prefix(self) -> bool: ...
    @inject_host_prefix.setter
    def inject_host_prefix(self, value: bool) -> None: ...



"""
Type annotations for botocore.tokens module.

Copyright 2025 Vlad Emelianov
"""

import datetime
import logging
from typing import Any, Callable, Iterable, NamedTuple

from botocore.session import Session
from botocore.utils import JSONFileCache

logger: logging.Logger

def create_token_resolver(session: Session) -> TokenProviderChain: ...

class FrozenAuthToken(NamedTuple):
    token: str
    expiration: datetime.datetime | None = ...

class DeferredRefreshableToken:
    def __init__(
        self,
        method: Any,
        refresh_using: Callable[[], FrozenAuthToken],
        time_fetcher: Callable[[], datetime.datetime] = ...,
    ) -> None: ...
    def get_frozen_token(self) -> FrozenAuthToken: ...

class TokenProviderChain:
    def __init__(self, providers: Iterable[Any] | None = ...) -> None: ...
    def load_token(self) -> DeferredRefreshableToken: ...

class SSOTokenProvider:
    METHOD: str = ...
    DEFAULT_CACHE_CLS: type[JSONFileCache] = ...

    def __init__(
        self,
        session: Session,
        cache: JSONFileCache | None = ...,
        time_fetcher: Callable[[], datetime.datetime] = ...,
        profile_name: str | None = ...,
    ) -> None: ...
    def load_token(self) -> DeferredRefreshableToken: ...



"""
Type annotations for botocore.validate module.

Copyright 2025 Vlad Emelianov
"""

from typing import Any, Iterable, Mapping

from botocore.exceptions import ParamValidationError as ParamValidationError
from botocore.model import OperationModel, Shape
from botocore.serialize import Serializer
from botocore.utils import is_json_value_header as is_json_value_header
from botocore.utils import parse_to_aware_datetime as parse_to_aware_datetime

def validate_parameters(params: Mapping[str, Any], shape: Shape) -> None: ...
def type_check(valid_types: Iterable[Any]) -> Any: ...
def range_check(
    name: str, value: Any, shape: Shape, error_type: Any, errors: ValidationErrors
) -> None: ...

class ValidationErrors:
    def __init__(self) -> None: ...
    def has_errors(self) -> bool: ...
    def generate_report(self) -> str: ...
    def report(self, name: str, reason: str, **kwargs: Any) -> None: ...

class ParamValidator:
    def validate(self, params: Mapping[str, Any], shape: Shape) -> Any: ...

class ParamValidationDecorator:
    def __init__(self, param_validator: ParamValidator, serializer: Serializer) -> None: ...
    def serialize_to_request(
        self, parameters: Iterable[Any], operation_model: OperationModel
    ) -> Any: ...



"""
Type annotations for botocore.retryhandler module.

Copyright 2025 Vlad Emelianov
"""

from logging import Logger
from typing import Any, Callable, Iterable, Mapping

from botocore.config import Config
from botocore.exceptions import BotoCoreError
from botocore.exceptions import ChecksumError as ChecksumError
from botocore.exceptions import ConnectionClosedError as ConnectionClosedError
from botocore.exceptions import ConnectionError as ConnectionError
from botocore.exceptions import EndpointConnectionError as EndpointConnectionError
from botocore.exceptions import ReadTimeoutError as ReadTimeoutError

logger: Logger = ...

EXCEPTION_MAP: dict[str, list[type[BotoCoreError]]]

def delay_exponential(base: float, growth_factor: float, attempts: int) -> float: ...
def create_exponential_delay_function(
    base: float, growth_factor: float
) -> Callable[[int], float]: ...
def create_retry_handler(config: Config, operation_name: str | None = ...) -> RetryHandler: ...
def create_retry_action_from_config(
    config: Config, operation_name: str | None = ...
) -> Callable[[int], float]: ...
def create_checker_from_retry_config(
    config: Config, operation_name: str | None = ...
) -> MaxAttemptsDecorator: ...

class RetryHandler:
    def __init__(self, checker: BaseChecker, action: Callable[[int], float]) -> None: ...
    def __call__(
        self, attempts: int, response: Mapping[str, Any], caught_exception: Exception, **kwargs: Any
    ) -> Any: ...

class BaseChecker:
    def __call__(
        self, attempt_number: int, response: Mapping[str, Any], caught_exception: Exception
    ) -> bool: ...

class MaxAttemptsDecorator(BaseChecker):
    def __init__(
        self,
        checker: BaseChecker,
        max_attempts: int,
        retryable_exceptions: Iterable[type[Exception]] | None = ...,
    ) -> None: ...
    # FIXME: Signature of "__call__" incompatible with supertype "BaseChecker"
    def __call__(  # type: ignore [override]
        self,
        attempt_number: int,
        response: Mapping[str, Any],
        caught_exception: Exception,
        retries_context: Any,
    ) -> bool: ...

class HTTPStatusCodeChecker(BaseChecker):
    def __init__(self, status_code: int) -> None: ...

class ServiceErrorCodeChecker(BaseChecker):
    def __init__(self, status_code: int, error_code: str) -> None: ...

class MultiChecker(BaseChecker):
    def __init__(self, checkers: Iterable[BaseChecker]) -> None: ...

class CRC32Checker(BaseChecker):
    def __init__(self, header: str) -> None: ...

class ExceptionRaiser(BaseChecker): ...



"""
Type annotations for botocore.args module.

Copyright 2025 Vlad Emelianov
"""

from logging import Logger
from typing import Any, Literal, Mapping, TypedDict

from botocore.client import ClientEndpointBridge
from botocore.config import Config as Config
from botocore.configprovider import ConfigValueStore
from botocore.credentials import Credentials
from botocore.endpoint import Endpoint
from botocore.endpoint import EndpointCreator as EndpointCreator
from botocore.errorfactory import ClientExceptionsFactory
from botocore.hooks import BaseEventHooks
from botocore.loaders import Loader
from botocore.model import ServiceModel
from botocore.parsers import ResponseParser, ResponseParserFactory
from botocore.serialize import BaseRestSerializer
from botocore.signers import RequestSigner as RequestSigner
from botocore.useragent import UserAgentString

logger: Logger = ...

VALID_REGIONAL_ENDPOINTS_CONFIG: list[str] = ...
LEGACY_GLOBAL_STS_REGIONS: list[str] = ...
USERAGENT_APPID_MAXLEN: int = ...
VALID_REQUEST_CHECKSUM_CALCULATION_CONFIG: tuple[str, ...] = ...
VALID_RESPONSE_CHECKSUM_VALIDATION_CONFIG: tuple[str, ...] = ...
PRIORITY_ORDERED_SUPPORTED_PROTOCOLS: tuple[str, ...] = ...
VALID_ACCOUNT_ID_ENDPOINT_MODE_CONFIG: tuple[str, ...] = ...

class _GetClientArgsTypeDef(TypedDict):
    serializer: BaseRestSerializer
    endpoint: Endpoint
    response_parser: ResponseParser
    event_emitter: BaseEventHooks
    request_signer: RequestSigner
    service_model: ServiceModel
    loader: Loader
    client_config: Config
    partition: str | None
    exceptions_factory: ClientExceptionsFactory

class ClientArgsCreator:
    def __init__(
        self,
        event_emitter: BaseEventHooks,
        user_agent: str,
        response_parser_factory: ResponseParserFactory,
        loader: Loader,
        exceptions_factory: ClientExceptionsFactory,
        config_store: ConfigValueStore,
        user_agent_creator: UserAgentString | None = ...,
    ) -> None: ...
    def get_client_args(
        self,
        service_model: ServiceModel,
        region_name: str,
        is_secure: bool,
        endpoint_url: str | None,
        verify: str | bool | None,
        credentials: Any | None,
        scoped_config: Mapping[str, Any] | None,
        client_config: Config | None,
        endpoint_bridge: ClientEndpointBridge,
        auth_token: str | None = ...,
        endpoints_ruleset_data: Mapping[str, Any] | None = ...,
        partition_data: Mapping[str, Any] | None = ...,
    ) -> _GetClientArgsTypeDef: ...
    def compute_client_args(
        self,
        service_model: ServiceModel,
        client_config: Config | None,
        endpoint_bridge: ClientEndpointBridge,
        region_name: str,
        endpoint_url: str,
        is_secure: bool,
        scoped_config: Mapping[str, Any] | None,
    ) -> Any: ...
    def compute_s3_config(self, client_config: Config | None) -> dict[str, Any]: ...
    def compute_endpoint_resolver_builtin_defaults(
        self,
        region_name: str,
        service_name: str,
        s3_config: Mapping[str, Any],
        endpoint_bridge: ClientEndpointBridge,
        client_endpoint_url: str,
        legacy_endpoint_url: str,
        credentials: Credentials | None,
        account_id_endpoint_mode: Literal["preferred", "disabled", "required"] | None,
    ) -> dict[str, Any]: ...



"""
Type annotations for botocore.serialize module.

Copyright 2025 Vlad Emelianov
"""

from typing import Any, Mapping, Pattern

from botocore.compat import formatdate as formatdate
from botocore.model import OperationModel
from botocore.utils import is_json_value_header as is_json_value_header
from botocore.utils import parse_to_aware_datetime as parse_to_aware_datetime
from botocore.utils import percent_encode as percent_encode

DEFAULT_TIMESTAMP_FORMAT: str
ISO8601: str
ISO8601_MICRO: str
HOST_PREFIX_RE: Pattern[str]

def create_serializer(protocol_name: str, include_validation: bool = ...) -> Any: ...

class Serializer:
    DEFAULT_METHOD: str = ...
    MAP_TYPE: type[dict[str, Any]] = ...
    DEFAULT_ENCODING: str = ...
    def serialize_to_request(
        self, parameters: Mapping[str, Any], operation_model: OperationModel
    ) -> dict[str, Any]: ...

class QuerySerializer(Serializer):
    TIMESTAMP_FORMAT: str = ...
    def serialize_to_request(
        self, parameters: Mapping[str, Any], operation_model: OperationModel
    ) -> dict[str, Any]: ...

class EC2Serializer(QuerySerializer): ...

class JSONSerializer(Serializer):
    TIMESTAMP_FORMAT: str = ...
    def serialize_to_request(
        self, parameters: Mapping[str, Any], operation_model: OperationModel
    ) -> dict[str, Any]: ...

class CBORSerializer(Serializer):
    UNSIGNED_INT_MAJOR_TYPE: int = ...
    NEGATIVE_INT_MAJOR_TYPE: int = ...
    BLOB_MAJOR_TYPE: int = ...
    STRING_MAJOR_TYPE: int = ...
    LIST_MAJOR_TYPE: int = ...
    MAP_MAJOR_TYPE: int = ...
    TAG_MAJOR_TYPE: int = ...
    FLOAT_AND_SIMPLE_MAJOR_TYPE: int = ...

class BaseRestSerializer(Serializer):
    QUERY_STRING_TIMESTAMP_FORMAT: str = ...
    HEADER_TIMESTAMP_FORMAT: str = ...
    KNOWN_LOCATIONS: list[str] = ...
    def serialize_to_request(
        self, parameters: Mapping[str, Any], operation_model: OperationModel
    ) -> dict[str, Any]: ...

class BaseRpcV2Serializer(Serializer): ...
class RestJSONSerializer(BaseRestSerializer, JSONSerializer): ...

class RestXMLSerializer(BaseRestSerializer):
    TIMESTAMP_FORMAT: str = ...

class RpcV2CBORSerializer(BaseRpcV2Serializer, CBORSerializer):
    TIMESTAMP_FORMAT: str = ...

SERIALIZERS: dict[str, Serializer]



"""
Type annotations for botocore.exceptions module.

Copyright 2025 Vlad Emelianov
"""

from typing import IO, Any, Iterable, Mapping, TypedDict

import requests
from urllib3.exceptions import ReadTimeoutError as _ReadTimeoutError

class _ClientErrorResponseError(TypedDict, total=False):
    Code: str
    Message: str

class _ResponseMetadataTypeDef(TypedDict):
    RequestId: str
    HostId: str
    HTTPStatusCode: int
    HTTPHeaders: dict[str, Any]
    RetryAttempts: int

class _AttributeMapTypeDef(TypedDict, total=False):
    key: str
    value: Any

class _CancellationReasonTypeDef(TypedDict, total=False):
    Code: str
    Message: str
    Item: _AttributeMapTypeDef

class _ClientErrorResponseTypeDef(TypedDict, total=False):
    Status: str
    StatusReason: str
    Error: _ClientErrorResponseError
    ResponseMetadata: _ResponseMetadataTypeDef
    CancellationReasons: list[_CancellationReasonTypeDef]

class BotoCoreError(Exception):
    fmt: str
    def __init__(self, **kwargs: Any) -> None:
        self.kwargs: Mapping[str, Any]

class _DataNotFoundErrorKwargs(TypedDict):
    data_path: str

class DataNotFoundError(BotoCoreError):
    def __init__(self, *, data_path: str = ..., **kwargs: Any) -> None:
        self.kwargs: _DataNotFoundErrorKwargs

class _UnknownServiceErrorKwargs(_DataNotFoundErrorKwargs):
    service_name: str
    known_service_names: Iterable[str]

class UnknownServiceError(DataNotFoundError):
    def __init__(
        self, *, service_name: str = ..., known_service_names: Iterable[str] = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _UnknownServiceErrorKwargs

class _UnknownRegionErrorKwargs(TypedDict):
    service_name: str
    error_msg: str

class UnknownRegionError(BotoCoreError):
    def __init__(self, *, region_name: str = ..., error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownRegionErrorKwargs

class _ApiVersionNotFoundErrorKwargs(TypedDict):
    service_name: str
    api_version: str

class ApiVersionNotFoundError(BotoCoreError):
    def __init__(self, *, data_path: str = ..., api_version: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ApiVersionNotFoundErrorKwargs

class _HTTPClientErrorKwargs(TypedDict):
    error: Exception

class HTTPClientError(BotoCoreError):
    def __init__(self, request: Any = ..., response: Any = ..., **kwargs: Any) -> None:
        self.kwargs: _HTTPClientErrorKwargs

class _ConnectionErrorKwargs(TypedDict):
    error: Exception

class ConnectionError(BotoCoreError):
    def __init__(self, *, error: Exception = ..., **kwargs: Any) -> None:
        self.kwargs: _ConnectionErrorKwargs

class _InvalidIMDSEndpointErrorKwargs(TypedDict):
    endpoint: str

class InvalidIMDSEndpointError(BotoCoreError):
    def __init__(self, *, endpoint: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidIMDSEndpointErrorKwargs

class _InvalidIMDSEndpointModeErrorKwargs(TypedDict):
    mode: str
    valid_modes: Iterable[str]

class InvalidIMDSEndpointModeError(BotoCoreError):
    def __init__(self, *, mode: str = ..., valid_modes: Iterable[str], **kwargs: Any) -> None:
        self.kwargs: _InvalidIMDSEndpointModeErrorKwargs

class _EndpointURLErrorKwargs(_ConnectionErrorKwargs):
    endpoint_url: str

class EndpointConnectionError(ConnectionError):
    def __init__(self, *, endpoint_url: str = ..., **kwargs: Any) -> None:
        self.kwargs: _EndpointURLErrorKwargs

class _SSLErrorKwargs(TypedDict):
    endpoint_url: str
    error: Exception

class SSLError(ConnectionError, requests.exceptions.SSLError):
    def __init__(self, *, endpoint_url: str = ..., error: Exception = ..., **kwargs: Any) -> None:
        self.kwargs: _SSLErrorKwargs

class ConnectionClosedError(HTTPClientError):
    def __init__(
        self, request: Any = ..., response: Any = ..., *, endpoint_url: str = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _EndpointURLErrorKwargs

class ReadTimeoutError(HTTPClientError, requests.exceptions.ReadTimeout, _ReadTimeoutError):
    def __init__(
        self, request: Any = ..., response: Any = ..., *, endpoint_url: str = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _EndpointURLErrorKwargs
        self.request: Any
        self.response: Any

class ConnectTimeoutError(ConnectionError, requests.exceptions.ConnectTimeout):
    def __init__(self, *, endpoint_url: str = ..., **kwargs: Any) -> None:
        self.kwargs: _EndpointURLErrorKwargs

class _ProxyConnectionErrorKwargs(_ConnectionErrorKwargs):
    proxy_url: str

class ProxyConnectionError(ConnectionError, requests.exceptions.ProxyError):
    def __init__(self, *, proxy_url: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ProxyConnectionErrorKwargs

class _ResponseStreamingErrorKwargs(TypedDict):
    error: Any

class ResponseStreamingError(HTTPClientError):
    def __init__(
        self, request: Any = ..., response: Any = ..., *, error: Any = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _ResponseStreamingErrorKwargs

class NoCredentialsError(BotoCoreError): ...
class NoAuthTokenError(BotoCoreError): ...

class _TokenRetrievalErrorKwargs(TypedDict):
    provider: str
    error_msg: str

class TokenRetrievalError(BotoCoreError):
    def __init__(self, *, provider: str = ..., error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _TokenRetrievalErrorKwargs

class _PartialCredentialsErrorKwargs(TypedDict):
    provider: str
    cred_var: str

class PartialCredentialsError(BotoCoreError):
    def __init__(self, *, provider: str = ..., cred_var: str = ..., **kwargs: Any) -> None:
        self.kwargs: _PartialCredentialsErrorKwargs

class _CredentialRetrievalErrorKwargs(TypedDict):
    provider: str

class CredentialRetrievalError(BotoCoreError):
    def __init__(self, *, provider: str = ..., **kwargs: Any) -> None:
        self.kwargs: _CredentialRetrievalErrorKwargs

class _UnknownSignatureVersionErrorKwargs(TypedDict):
    signature_version: str

class UnknownSignatureVersionError(BotoCoreError):
    def __init__(self, *, signature_version: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownSignatureVersionErrorKwargs

class _ServiceNotInRegionErrorKwargs(TypedDict):
    service_name: str
    region_name: str

class ServiceNotInRegionError(BotoCoreError):
    def __init__(self, *, service_name: str = ..., region_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ServiceNotInRegionErrorKwargs

class BaseEndpointResolverError(BotoCoreError): ...
class NoRegionError(BaseEndpointResolverError): ...

class _EndpointVariantErrorKwargs(TypedDict):
    tags: Iterable[str]

class EndpointVariantError(BaseEndpointResolverError):
    def __init__(self, *, tags: Iterable[str] = ..., **kwargs: Any) -> None:
        self.kwargs: _EndpointVariantErrorKwargs

class _UnknownEndpointErrorKwargs(TypedDict):
    service_name: str
    region_name: str

class UnknownEndpointError(BaseEndpointResolverError, ValueError):
    def __init__(self, *, service_name: str = ..., region_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownEndpointErrorKwargs

class _UnknownFIPSEndpointErrorKwargs(TypedDict):
    service_name: str
    region_name: str

class UnknownFIPSEndpointError(BaseEndpointResolverError):
    def __init__(self, *, service_name: str = ..., region_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownFIPSEndpointErrorKwargs

class _ProfileNotFoundKwargs(TypedDict):
    profile: str

class ProfileNotFound(BotoCoreError):
    def __init__(self, *, profile: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ProfileNotFoundKwargs

class _ConfigParseErrorKwargs(TypedDict):
    path: str

class ConfigParseError(BotoCoreError):
    def __init__(self, *, path: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ConfigParseErrorKwargs

class _ConfigNotFoundKwargs(TypedDict):
    path: str

class ConfigNotFound(BotoCoreError):
    def __init__(self, *, path: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ConfigNotFoundKwargs

class _MissingParametersErrorKwargs(TypedDict):
    object: Any
    missing: Iterable[str]

class MissingParametersError(BotoCoreError):
    def __init__(self, *, object: Any = ..., missing: Iterable[str] = ..., **kwargs: Any) -> None:
        self.kwargs: _MissingParametersErrorKwargs

class _ValidationErrorKwargs(TypedDict):
    value: Any
    param: str
    type_name: str

class ValidationError(BotoCoreError):
    def __init__(
        self, *, value: Any = ..., param: str = ..., type_name: str = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _ValidationErrorKwargs

class _ParamValidationErrorKwargs(TypedDict):
    report: str

class ParamValidationError(BotoCoreError):
    def __init__(self, *, report: str = ..., **kwargs: Any) -> None:
        self.kwargs: _ParamValidationErrorKwargs

class _UnknownKeyErrorKwargs(_ValidationErrorKwargs):
    choices: Iterable[Any]

class UnknownKeyError(ValidationError):
    def __init__(
        self, *, value: Any = ..., param: str = ..., choices: Iterable[Any] = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _UnknownKeyErrorKwargs

class _RangeErrorKwargs(_ValidationErrorKwargs):
    min_value: Any
    max_value: Any

class RangeError(ValidationError):
    def __init__(
        self,
        *,
        value: Any = ...,
        param: str = ...,
        min_value: Any = ...,
        max_value: Any = ...,
        **kwargs: Any,
    ) -> None:
        self.kwargs: _RangeErrorKwargs

class _UnknownParameterErrorKwargs(_ValidationErrorKwargs):
    name: str
    operation: str
    choices: Iterable[str]

class UnknownParameterError(ValidationError):
    def __init__(
        self, *, name: str = ..., operation: str = ..., choices: Iterable[str] = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _UnknownParameterErrorKwargs

class _InvalidRegionErrorKwargs(_ValidationErrorKwargs):
    region_name: str

class InvalidRegionError(ValidationError, ValueError):
    def __init__(self, *, region_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidRegionErrorKwargs

class _AliasConflictParameterErrorKwargs(_ValidationErrorKwargs):
    original: str
    alias: str
    operation: str

class AliasConflictParameterError(ValidationError):
    def __init__(
        self, *, original: str = ..., alias: str = ..., operation: str = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _AliasConflictParameterErrorKwargs

class _UnknownServiceStyleKwargs(TypedDict):
    service_style: str

class UnknownServiceStyle(BotoCoreError):
    def __init__(self, *, service_style: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownServiceStyleKwargs

class _PaginationErrorKwargs(TypedDict):
    message: str

class PaginationError(BotoCoreError):
    def __init__(self, *, message: str = ..., **kwargs: Any) -> None:
        self.kwargs: _PaginationErrorKwargs

class _OperationNotPageableErrorKwargs(TypedDict):
    operation_name: str

class OperationNotPageableError(BotoCoreError):
    def __init__(self, *, operation_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _OperationNotPageableErrorKwargs

class _ChecksumErrorKwargs(TypedDict):
    checksum_type: str
    expected_checksum: str
    actual_checksum: str

class ChecksumError(BotoCoreError):
    def __init__(
        self,
        *,
        checksum_type: str = ...,
        expected_checksum: str = ...,
        actual_checksum: str = ...,
        **kwargs: Any,
    ) -> None:
        self.kwargs: _ChecksumErrorKwargs

class _UnseekableStreamErrorKwargs(TypedDict):
    stream_object: IO[Any]

class UnseekableStreamError(BotoCoreError):
    def __init__(self, *, stream_object: IO[Any] = ..., **kwargs: Any) -> None:
        self.kwargs: _UnseekableStreamErrorKwargs

class WaiterError(BotoCoreError):
    def __init__(self, name: str, reason: str, last_response: _ClientErrorResponseTypeDef) -> None:
        self.last_response: _ClientErrorResponseTypeDef

class _IncompleteReadErrorKwargs(TypedDict):
    actual_bytes: int
    expected_bytes: int

class IncompleteReadError(BotoCoreError):
    def __init__(
        self, *, actual_bytes: int = ..., expected_bytes: int = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _IncompleteReadErrorKwargs

class _InvalidExpressionErrorKwargs(TypedDict):
    expression: str

class InvalidExpressionError(BotoCoreError):
    def __init__(self, *, expression: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidExpressionErrorKwargs

class _UnknownCredentialErrorKwargs(TypedDict):
    name: str

class UnknownCredentialError(BotoCoreError):
    def __init__(self, *, name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownCredentialErrorKwargs

class _WaiterConfigErrorKwargs(TypedDict):
    error_msg: str

class WaiterConfigError(BotoCoreError):
    def __init__(self, *, error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _WaiterConfigErrorKwargs

class _UnknownClientMethodErrorKwargs(TypedDict):
    method_name: str

class UnknownClientMethodError(BotoCoreError):
    def __init__(self, *, method_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnknownClientMethodErrorKwargs

class _UnsupportedSignatureVersionErrorKwargs(TypedDict):
    signature_version: str

class UnsupportedSignatureVersionError(BotoCoreError):
    def __init__(self, *, signature_version: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedSignatureVersionErrorKwargs

class ClientError(Exception):
    MSG_TEMPLATE: str
    def __init__(self, error_response: _ClientErrorResponseTypeDef, operation_name: str) -> None:
        self.response: _ClientErrorResponseTypeDef
        self.operation_name: str

class EventStreamError(ClientError): ...
class UnsupportedTLSVersionWarning(Warning): ...
class ImminentRemovalWarning(Warning): ...

class _InvalidDNSNameErrorKwargs(TypedDict):
    bucket_name: str

class InvalidDNSNameError(BotoCoreError):
    def __init__(self, *, bucket_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidDNSNameErrorKwargs

class _InvalidS3AddressingStyleErrorKwargs(TypedDict):
    s3_addressing_style: str

class InvalidS3AddressingStyleError(BotoCoreError):
    def __init__(self, *, s3_addressing_style: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidS3AddressingStyleErrorKwargs

class _UnsupportedS3ArnErrorKwargs(TypedDict):
    arn: str

class UnsupportedS3ArnError(BotoCoreError):
    def __init__(self, *, arn: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedS3ArnErrorKwargs

class _UnsupportedS3ControlArnErrorKwargs(TypedDict):
    arn: str
    msg: str

class UnsupportedS3ControlArnError(BotoCoreError):
    def __init__(self, *, arn: str = ..., msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedS3ControlArnErrorKwargs

class _InvalidHostLabelErrorKwargs(TypedDict):
    label: str

class InvalidHostLabelError(BotoCoreError):
    def __init__(self, *, label: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidHostLabelErrorKwargs

class _UnsupportedOutpostResourceErrorKwargs(TypedDict):
    resource_name: str

class UnsupportedOutpostResourceError(BotoCoreError):
    def __init__(self, *, resource_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedOutpostResourceErrorKwargs

class _UnsupportedS3ErrorKwargs(TypedDict):
    msg: str

class UnsupportedS3ConfigurationError(BotoCoreError):
    def __init__(self, *, msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedS3ErrorKwargs

class UnsupportedS3AccesspointConfigurationError(BotoCoreError):
    def __init__(self, *, msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedS3ErrorKwargs

class _InvalidEndpointDiscoveryConfigurationErrorKwargs(TypedDict):
    config_value: str

class InvalidEndpointDiscoveryConfigurationError(BotoCoreError):
    def __init__(self, *, config_value: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidEndpointDiscoveryConfigurationErrorKwargs

class UnsupportedS3ControlConfigurationError(BotoCoreError):
    def __init__(self, *, msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _UnsupportedS3ErrorKwargs

class _InvalidRetryConfigurationErrorKwargs(TypedDict):
    retry_config_option: str

class InvalidRetryConfigurationError(BotoCoreError):
    def __init__(self, *, retry_config_option: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidRetryConfigurationErrorKwargs

class _InvalidMaxRetryAttemptsErrorKwargs(_InvalidRetryConfigurationErrorKwargs):
    provided_max_attempts: int
    min_value: int

class InvalidMaxRetryAttemptsError(InvalidRetryConfigurationError):
    def __init__(
        self, *, provided_max_attempts: int = ..., min_value: int = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _InvalidMaxRetryAttemptsErrorKwargs

class _InvalidRetryModeErrorKwargs(_InvalidMaxRetryAttemptsErrorKwargs):
    provided_retry_mode: str

class InvalidRetryModeError(InvalidRetryConfigurationError):
    def __init__(self, *, provided_retry_mode: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidRetryModeErrorKwargs

class _InvalidS3UsEast1RegionalEndpointConfigErrorKwargs(TypedDict):
    s3_us_east_1_regional_endpoint_config: str

class InvalidS3UsEast1RegionalEndpointConfigError(BotoCoreError):
    def __init__(self, *, s3_us_east_1_regional_endpoint_config: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidS3UsEast1RegionalEndpointConfigErrorKwargs

class _InvalidSTSRegionalEndpointsConfigErrorKwargs(TypedDict):
    sts_regional_endpoints_config: str

class InvalidSTSRegionalEndpointsConfigError(BotoCoreError):
    def __init__(self, *, sts_regional_endpoints_config: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidSTSRegionalEndpointsConfigErrorKwargs

class _StubResponseErrorKwargs(TypedDict):
    operation_name: str

class StubResponseError(BotoCoreError):
    def __init__(self, *, operation_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _StubResponseErrorKwargs

class StubAssertionError(StubResponseError, AssertionError): ...
class UnStubbedResponseError(StubResponseError): ...

class _InvalidConfigErrorKwargs(TypedDict):
    error_msg: str

class InvalidConfigError(BotoCoreError):
    def __init__(self, *, error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidConfigErrorKwargs

class _InfiniteLoopConfigErrorKwargs(_InvalidConfigErrorKwargs):
    source_profile: str
    visited_profiles: Iterable[str]

class InfiniteLoopConfigError(InvalidConfigError):
    def __init__(
        self, *, source_profile: str = ..., visited_profiles: Iterable[str] = ..., **kwargs: Any
    ) -> None:
        self.kwargs: _InfiniteLoopConfigErrorKwargs

class RefreshWithMFAUnsupportedError(BotoCoreError): ...
class MD5UnavailableError(BotoCoreError): ...

class _MissingDependencyExceptionKwargs(TypedDict):
    msg: str

class MissingDependencyException(BotoCoreError):
    def __init__(self, *, msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _MissingDependencyExceptionKwargs

class _MetadataRetrievalErrorKwargs(TypedDict):
    error_msg: str

class MetadataRetrievalError(BotoCoreError):
    def __init__(self, *, error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _MetadataRetrievalErrorKwargs

class UndefinedModelAttributeError(Exception): ...

class _MissingServiceIdErrorKwargs(TypedDict):
    service_name: str

class MissingServiceIdError(UndefinedModelAttributeError):
    fmt: str
    def __init__(self, *, service_name: str = ..., **kwargs: Any) -> None:
        self.kwargs: _MissingServiceIdErrorKwargs

class SSOError(BotoCoreError): ...

class _SSOTokenLoadErrorKwargs(TypedDict):
    error_msg: str

class SSOTokenLoadError(SSOError):
    def __init__(self, *, error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _SSOTokenLoadErrorKwargs

class UnauthorizedSSOTokenError(SSOError): ...
class CapacityNotAvailableError(BotoCoreError): ...
class InvalidProxiesConfigError(BotoCoreError): ...

class _InvalidDefaultsModeKwargs(TypedDict):
    mode: str
    valid_modes: Iterable[str]

class InvalidDefaultsMode(BotoCoreError):
    def __init__(self, *, mode: str = ..., valid_modes: Iterable[str] = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidDefaultsModeKwargs

class _AwsChunkedWrapperErrorKwargs(TypedDict):
    error_msg: str

class AwsChunkedWrapperError(BotoCoreError):
    def __init__(self, *, error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _AwsChunkedWrapperErrorKwargs

class _FlexibleChecksumErrorKwargs(TypedDict):
    error_msg: str

class FlexibleChecksumError(BotoCoreError):
    def __init__(self, *, error_msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _FlexibleChecksumErrorKwargs

class _InvalidEndpointConfigurationErrorKwargs(TypedDict):
    msg: str

class InvalidEndpointConfigurationError(BotoCoreError):
    def __init__(self, *, msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _InvalidEndpointConfigurationErrorKwargs

class _EndpointProviderErrorKwargs(TypedDict):
    msg: str

class EndpointProviderError(BotoCoreError):
    def __init__(self, *, msg: str = ..., **kwargs: Any) -> None:
        self.kwargs: _EndpointProviderErrorKwargs

class EndpointResolutionError(EndpointProviderError): ...
class UnknownEndpointResolutionBuiltInName(EndpointProviderError): ...
class InvalidChecksumConfigError(BotoCoreError): ...
class UnsupportedServiceProtocolsError(BotoCoreError): ...



"""
Type annotations for botocore.docs.paginator module.

Copyright 2025 Vlad Emelianov
"""

from typing import Any

from botocore.client import BaseClient
from botocore.docs.bcdoc.restdoc import DocumentStructure
from botocore.hooks import BaseEventHooks
from botocore.model import ServiceModel

class PaginatorDocumenter:
    def __init__(
        self, client: BaseClient, service_paginator_model: ServiceModel, root_docs_path: str
    ) -> None: ...
    def document_paginators(self, section: DocumentStructure) -> None: ...

def document_paginate_method(
    section: DocumentStructure,
    paginator_name: str,
    event_emitter: BaseEventHooks,
    service_model: ServiceModel,
    paginator_config: Any,
    include_signature: bool = ...,
) -> None: ...
